#define G_LOG_DOMAIN "gpod-feed-channel"

#include "gpod-feed-channel.h"

#include <glib.h>

/**
 * SECTION:gpod-feed-channel
 * @short_description: a container to hold a RSS channel in a #GpodFeedChannel
 *
 * #GpodFeedChannel is the representation of a RSS channel in a valid RSS feed. It
 * contains information about the channel like title, link, publish date, but
 * more importantly all the RSS items; which are stored as #GpodItems.
 *
 * Since #GpodFeedChannel can contain many #GpodItems it can be expensive and for
 * this reason they are reference counted. You can control the lifetime of a
 * #GpodFeedChannel using gpod_feed_channel_ref() and gpod_feed_channel_unref().
 */

G_DEFINE_BOXED_TYPE (GpodFeedChannel, gpod_feed_channel, gpod_feed_channel_ref, gpod_feed_channel_unref)

/**
 * gpod_feed_channel_new: (constructor)
 *
 * Creates a new #GpodFeedChannel, a container to hold a RSS channel in a #GpodFeedChannel.
 *
 * Returns: (transfer full): the newly created #GpodFeedChannel
 */
GpodFeedChannel *
gpod_feed_channel_new (void)
{
    GpodFeedChannel *object;

    object = g_slice_new0 (GpodFeedChannel);
    object->ref_count = 1;

    object->guid = NULL;
    object->title = NULL;
    object->description = NULL;
    object->link = NULL;
    object->publish_date = NULL;
    object->updated = NULL;
    object->image = NULL;

    object->items = g_ptr_array_new ();

    return object;
}

/**
 * gpod_feed_channel_ref:
 * @object: a #GpodFeedChannel
 *
 * Increase by one the reference count of a #GpodFeedChannel.
 *
 * Returns: (transfer none): the passed #GpodFeedChannel, with the reference count
 *                           increased by one.
 */
GpodFeedChannel *
gpod_feed_channel_ref (GpodFeedChannel *object)
{
    g_return_val_if_fail (object != NULL, NULL);
    g_return_val_if_fail (object->ref_count > 0, NULL);

    object->ref_count++;

    return object;
}

/**
 * gpod_feed_channel_unref:
 * @object: a #GpodFeedChannel
 *
 * Decreases by one the reference count of a #GpodFeedChannel. If the
 * reference count reaches zero, the object is destroyed and all
 * its allocated resources are freed.
 */
void
gpod_feed_channel_unref (GpodFeedChannel *object)
{
    g_return_if_fail (object != NULL);
    g_return_if_fail (object->ref_count > 0);

    if (--object->ref_count == 0)
    {
        guint i;
/*
        if (object->guid != NULL)
            g_free (object->guid);

        if (object->title != NULL)
            g_free (object->title);

        if (object->description != NULL)
            g_free (object->description);

        if (object->link != NULL)
            g_free (object->link);

        if (object->publish_date != NULL)
            g_free (object->publish_date);

        if (object->updated != NULL)
            g_free (object->updated);

        if (object->image != NULL)
            g_free (object->image);
*/
        for (i = 0; i < object->items->len; i++)
            gpod_feed_item_unref (g_ptr_array_index (object->items, i));

        g_ptr_array_free (object->items, TRUE);
/*
        object->guid = NULL;
        object->title = NULL;
        object->description = NULL;
        object->link = NULL;
        object->publish_date = NULL;
        object->updated = NULL;
        object->image = NULL;
        object->items = NULL;
*/
        g_slice_free (GpodFeedChannel, object);
    }
}
