#define G_LOG_DOMAIN "gpod-feed-manager"

#include "config.h"

#include "gpod-tracker.h"
#include "gpod-feed-manager.h"

#include <glib.h>
#include <glib-object.h>

/**
 * SECTION:gpod-feed-manager
 * @title: GpodFeedManager
 * @short_description: manages feeds and saving of information to and from the
 *                     tracker database.
 * @include: gpod-feed-manager.h
 *
 * The #GpodFeedManager is object to manage the fetching, saving, and retrevial of
 * feeds and their items.
 **/

typedef struct _GpodFeedManagerQueryData
{
    GpodFeedManagerGetFeeds  callback;
    gpointer                 user_data;
    GpodFeedManager         *self;
} GpodFeedManagerQueryData;

typedef struct _GpodFeedManagerPrivate
{
    GpodTracker *tracker;
} GpodFeedManagerPrivate;

struct _GpodFeedManager
{
    GObject parent;
};

G_DEFINE_TYPE_WITH_PRIVATE (GpodFeedManager, gpod_feed_manager, G_TYPE_OBJECT)

enum
{
    LOADING,
    LOADING_FINISHED,
    DATABASE_ERROR,
    NO_RESULTS,
    NUM_SIGNALS
};

static guint signals[NUM_SIGNALS] = { 0, };

void
gpod_feed_manager_add_feed_by_url (GpodFeedManager *self,
                                   gchar           *url)
{
    g_return_if_fail (GPOD_IS_FEED_MANAGER (self));

    GpodFeedManagerPrivate *priv = gpod_feed_manager_get_instance_private (self);
    GString *query = g_string_new (NULL);

    g_string_printf (query,
                     "INSERT { "
                     "    _:FeedSettings a mfo:FeedSettings ; "
                     "        mfo:updateInterval %i . "
                     "    _:Feed a nie:DataObject, mfo:FeedChannel ; "
                     "        mfo:feedSettings _:FeedSettings ; "
                     "        nie:url <%s> "
                     "} ",
                     30, url);

    gpod_tracker_query_update (priv->tracker, query->str, NULL);

    g_string_free (query, TRUE);
}

void
gpod_feed_manager_remove_feed_by_url (GpodFeedManager *self,
                                      gchar           *url)
{
    g_return_if_fail (GPOD_IS_FEED_MANAGER (self));
}


static void
get_feeds_with_cursor_cb (GObject      *object,
                          GAsyncResult *result,
                          gpointer      user_data)
{
	TrackerSparqlCursor *cursor = TRACKER_SPARQL_CURSOR (object);
    GpodTrackerQuery *query_data = user_data;
    GpodFeedManagerQueryData *internal_user_data = query_data->user_data;
    GpodFeedManager *self = internal_user_data->self;
	GError *error = NULL;
	gboolean results;

	results = gpod_tracker_query_cursor_finish (cursor, result, &error);

    if (error != NULL)
    {
        // Print warning to gui message
        g_signal_emit (self, signals[DATABASE_ERROR], 0, NULL);
        g_warning ("Could not run cursor next: %s", error->message);
        g_error_free (error);
        return;
    }

    if (results)
    {
        g_print ("  %s\n", tracker_sparql_cursor_get_string (cursor, 0, NULL));
        g_print ("  %s\n", tracker_sparql_cursor_get_string (cursor, 1, NULL));
        g_print ("  %s\n", tracker_sparql_cursor_get_string (cursor, 2, NULL));
        g_print ("\n");

        internal_user_data->callback (g_strdup (tracker_sparql_cursor_get_string (cursor, 0, NULL)),
                                      g_strdup (tracker_sparql_cursor_get_string (cursor, 1, NULL)),
                                      g_strdup (tracker_sparql_cursor_get_string (cursor, 2, NULL)),
                                      internal_user_data->user_data);

        gpod_tracker_query_cursor_next (cursor, query_data->cancellable, get_feeds_with_cursor_cb, query_data);
    }
    else
    {
        g_signal_emit (self, signals[LOADING_FINISHED], 0, NULL);

        //g_object_unref (object);
        //g_cancellable_cancel (query_data->cancellable);
        //g_object_unref (query_data->cancellable);
        //g_free (query_data);
    }
}

static void
get_feeds_with_cb (GObject      *object,
                   GAsyncResult *result,
                   gpointer      user_data)
{
    TrackerSparqlConnection *connection = TRACKER_SPARQL_CONNECTION (object);
    TrackerSparqlCursor *cursor = NULL;
    GpodTrackerQuery *query_data = user_data;
    GpodFeedManagerQueryData *internal_user_data = query_data->user_data;
    GpodFeedManager *self = internal_user_data->self;
    GError *error = NULL;

    cursor = gpod_tracker_query_finish (connection, result, query_data, &error);

    if (error != NULL)
    {
        // TODO: callback to error, print warning to gui message
        g_signal_emit (self, signals[DATABASE_ERROR], 0, NULL);
        g_warning ("Unable to get feed from Tracker database: %s", error->message);
        g_error_free (error);

        return;
    }

    if (!cursor)
    {
        // TODO: callback to error, print warning to gui message
        g_signal_emit (self, signals[NO_RESULTS], 0, NULL);
        g_print ("%s\n", "No results were found");

        return;
    }

    g_print ("Adding feeds...\n");
    gpod_tracker_query_cursor_next (cursor, query_data->cancellable, get_feeds_with_cursor_cb, query_data);
}

/**
 * gpod_feed_manager_get_feeds_with()
 * @self: a #GpodFeedManager
 * @cancellable: a #GCancellable
 * @callback: a #GpodFeedManagerGetFeeds callback function that each
 *            #TrackerSparqlCursor will call
 * @user_data: The object for @callback
 *
 */
void
gpod_feed_manager_get_feeds_with (GpodFeedManager         *self,
                                  GCancellable            *cancellable,
                                  GpodFeedManagerGetFeeds  callback,
                                  gpointer                 user_data)
{
    g_return_if_fail (GPOD_IS_FEED_MANAGER (self));

    GpodFeedManagerPrivate *priv = gpod_feed_manager_get_instance_private (self);
    GString *query = g_string_new (NULL);
    GpodFeedManagerQueryData *internal_user_data;

    g_signal_emit (self, signals[LOADING], 0, NULL);

/*
    g_string_printf (query,
                     "SELECT "
                     "    ?urn AS urn "
                     "    ?url AS url "
                     "    ?title AS title "
                     "    ?description AS description "
                     "    ?copyright AS copyright "
                     "    ?updated AS updated "
                     "    ?image AS image "
                     "    ?updateInterval AS updateInterval "
                     "WHERE { "
                     "    ?urn a mfo:FeedChannel ; "
                     "        nie:url ?url ; "
                     "        nie:title ?title ; "
                     "        nie:description ?description ; "
                     "        nie:copyright ?copyright ; "
                     "        nmo:lastMessageDate ?updated ; "
                     "        mfo:image ?image . "
                     "    OPTIONAL { "
                     "        ?urn a mfo:FeedChannel ;"
                     "            mfo:updateInterval ?updateInterval . "
                     "    } "
                     "} ORDER BY nie:title(?urn)");
*/
    g_string_printf (query,
                     "SELECT "
                     "    ?url "
                     "    ?title "
                     "    ?image "
                     "WHERE { "
                     "    ?urn a mfo:FeedChannel ; "
                     "        nie:url ?url ; "
                     "        nie:title ?title ; "
                     "        mfo:image ?image . "
                     "} ORDER BY nie:title(?urn)");

    internal_user_data = g_slice_new (GpodFeedManagerQueryData);
    internal_user_data->callback = callback;
    internal_user_data->user_data = user_data;
    internal_user_data->self = self;

    gpod_tracker_query (priv->tracker, query->str, cancellable, get_feeds_with_cb, internal_user_data);

    g_string_free (query, TRUE);
}

static void
on_tracker_database_error (GpodTracker     *gpod_tracker,
                           gchar           *error_message,
                           GpodFeedManager *self)
{
    g_return_if_fail (GPOD_IS_TRACKER (gpod_tracker));
    g_return_if_fail (GPOD_IS_FEED_MANAGER (self));

    // TODO: Emit database errot to DB
    g_signal_emit (self, signals[DATABASE_ERROR], 0, NULL);
}

/*
 * gpod_feed_manager_finalize:
 * @object: An instance of #GpodFeedManager
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_feed_manager_finalize (GObject *object)
{
    GpodFeedManager *self = GPOD_FEED_MANAGER (object);
    GpodFeedManagerPrivate *priv = gpod_feed_manager_get_instance_private (self);

    g_clear_object (&priv->tracker);

    G_OBJECT_CLASS (gpod_feed_manager_parent_class)->finalize(object);
}

/*
 * gpod_feed_manager_init:
 * @self: An instance of #GpodFeedManager
 *
 * Initializes various aspects of the #GpodFeedManager. Anything the #GpodFeedManager
 * would need before becoming presented should be done here.
 */
static void
gpod_feed_manager_init (GpodFeedManager *self)
{
    GpodFeedManagerPrivate *priv = gpod_feed_manager_get_instance_private (self);

    priv->tracker = gpod_tracker_new ();

    g_signal_connect (priv->tracker,
                      "database-error",
                      G_CALLBACK (on_tracker_database_error),
                      self);
}

/*
 * gpod_feed_manager_class_init:
 * @klass: An instance of #GpodFeedManagerClass
 *
 * Initializes the various virtural functions in #GpodFeedManagerClass. In
 * addtition it connects the various #GtkWidgets from the gresource file using
 * gtk_widget_class_set_template_from_resource().
 */
static void
gpod_feed_manager_class_init (GpodFeedManagerClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    signals[LOADING] =
        g_signal_new ("loading",
                      G_TYPE_FROM_CLASS (object_class),
                      G_SIGNAL_RUN_LAST,
                      0,
                      NULL,
                      NULL,
                      NULL,
                      G_TYPE_NONE,
                      0,
                      NULL);

    signals[LOADING_FINISHED] =
        g_signal_new ("loading-finished",
                      G_TYPE_FROM_CLASS (object_class),
                      G_SIGNAL_RUN_LAST,
                      0,
                      NULL,
                      NULL,
                      NULL,
                      G_TYPE_NONE,
                      0,
                      NULL);

    signals[DATABASE_ERROR] =
        g_signal_new ("database-error",
                      G_TYPE_FROM_CLASS (object_class),
                      G_SIGNAL_RUN_LAST,
                      0,
                      NULL,
                      NULL,
                      NULL,
                      G_TYPE_NONE,
                      0,
                      NULL);

    signals[NO_RESULTS] =
        g_signal_new ("no-results",
                      G_TYPE_FROM_CLASS (object_class),
                      G_SIGNAL_RUN_LAST,
                      0,
                      NULL,
                      NULL,
                      NULL,
                      G_TYPE_NONE,
                      0,
                      NULL);

    object_class->finalize = gpod_feed_manager_finalize;
}

/**
 * gpod_feed_manager_new:
 *
 * Creates a new #GpodFeedManager, a custom widget to show all feeds user is
 * subscribed to.
 *
 * ## Usage
 * |[
 * GpodFeedManager *manager;
 *
 * manager = gpod_feed_manager_new ();
 * ]|
 *
 * Returns: #GpodFeedManager
 */
GpodFeedManager *
gpod_feed_manager_new ()
{
    return g_object_new (GPOD_TYPE_FEED_MANAGER,
                         NULL);
}
