#define G_LOG_DOMAIN "gpod-window"

#include "config.h"

#include "gpod-library-view.h"
#include "gpod-feed-view.h"

#include "gpod-feed-manager.h"

#include "gpod-application.h"
#include "gpod-window.h"

#include <gtk/gtk.h>
#include <glib/gi18n.h>

/**
 * SECTION:gpod-window
 * @title: GpodWindow
 * @short_description: A #GtkApplicationWindow instance created for
 * #GpodApplication
 * @include: gpod-window.h
 * @see_also: #GtkApplicationWindow
 *
 * The #GpodApplication is an instance #GtkApplicationWindow, and acts as
 * the primary window for the current #GtkApplication.
 **/

typedef struct
{
    gint width;
    gint height;
    gint x;
    gint y;
    gboolean maximized;
    gboolean fullscreen;
} GpodWindowState;

typedef struct _GpodWindowPrivate
{
    GtkWidget *box;
    GtkWidget *headerbar;
    GtkWidget *overlay;
    GtkWidget *notification_loading;
    GtkWidget *notification_database_error;
    GtkWidget *notification_database_error_button_reload;
    GtkWidget *notification_database_error_button_close;
    GtkWidget *actionbar;
    GtkWidget *player;
    GtkStack *stack;

    GpodFeedManager *feed_manager;

    GpodLibraryView *library_view;
    GpodFeedView *feed_view;

    guint timeout_id;
} GpodWindowPrivate;

struct _GpodWindow
{
    GtkApplicationWindow parent_instance;

    GpodWindowState state;
};

#define STATE_TIMEOUT 100 /* ms */

G_DEFINE_TYPE_WITH_PRIVATE (GpodWindow, gpod_window, GTK_TYPE_APPLICATION_WINDOW)

enum {
    PROP_0,
    PROP_FEED_MANAGER,
    LAST_PROP
};

static GParamSpec *properties[LAST_PROP] = { NULL, };

/*
 * Library view
 * Load all feeds from database and with each feed run the callback
 * function, gpod_library_view_add_feed(), adding the feed to the
 * library_view in priv->stack.
 */
static void
load_library_view (GpodWindow *self)
{
    g_return_if_fail (GPOD_IS_WINDOW (self));

    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    gpod_feed_manager_get_feeds_with (GPOD_FEED_MANAGER (priv->feed_manager),
                                      NULL,
                                      gpod_library_view_add_feed,
                                      priv->library_view);
}

static void
hide_all_notification (GpodWindow *self)
{
    g_return_if_fail (GPOD_IS_WINDOW (self));

    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification_loading), FALSE);
    gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification_database_error), FALSE);
}

/*
 *******************************************************************************
 * Callbacks
 *******************************************************************************
 */
static void
on_notifcation_database_error_button_reload (GtkWidget  *widget,
                                             GpodWindow *self)
{
    g_return_if_fail (GTK_IS_WIDGET (widget));
    g_return_if_fail (GPOD_IS_WINDOW (self));

    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    // TODO: Remove old items
    /*
     * https://developer.gnome.org/gtk3/stable/GtkContainer.html
     * GList *children = gtk_container_get_children (GTK_CONTAINER (priv->library_view));
     * gtk_container_remove (GTK_CONTAINER (priv->library_view), child);
    gtk_container_forall (GTK_CONTAINER (priv->library_view),
                          g_object_unref,
                          NULL);
    */

    load_library_view (self);

    gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification_database_error), FALSE);
}

static void
on_notifcation_database_error_button_close (GtkWidget  *widget,
                                            GpodWindow *self)
{
    g_return_if_fail (GTK_IS_WIDGET (widget));
    g_return_if_fail (GPOD_IS_WINDOW (self));

    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification_database_error), FALSE);
}

static void
on_notifcation_loading_show (GpodFeedManager *feed_manager,
                             GpodWindow      *self)
{
    g_return_if_fail (GPOD_IS_FEED_MANAGER (feed_manager));
    g_return_if_fail (GPOD_IS_WINDOW (self));

    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    hide_all_notification (self);

    gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification_loading), TRUE);
}

static void
on_notifcation_loading_hide (GpodFeedManager *feed_manager,
                             GpodWindow      *self)
{
    g_return_if_fail (GPOD_IS_FEED_MANAGER (feed_manager));
    g_return_if_fail (GPOD_IS_WINDOW (self));

    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification_loading), FALSE);
}

static void
on_notifcation_database_error_show (GpodFeedManager *feed_manager,
                                    GpodWindow      *self)
{
    g_return_if_fail (GPOD_IS_FEED_MANAGER (feed_manager));
    g_return_if_fail (GPOD_IS_WINDOW (self));

    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    hide_all_notification (self);

    gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification_database_error), TRUE);
}

static void
on_notifcation_no_results (GpodFeedManager *feed_manager,
                           GpodWindow      *self)
{
    g_return_if_fail (GPOD_IS_FEED_MANAGER (feed_manager));
    g_return_if_fail (GPOD_IS_WINDOW (self));

    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    if (priv->stack && GTK_IS_STACK (priv->stack))
        gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "empty_view_feeds");
}

/*
 * on_close:
 * @action: A pointer to the object that originated the callback. In this
 * instance a #GSimpleAction, which will be added to the #GAction menu
 * @parameter: arg_data from the #GActionEntry
 * @user_data: User data set when the signal handler was connected, in this
 * instance it should be a #GtkApplication
 *
 * An action wrapper to close the current window.
 */
static void
on_close (GSimpleAction *action,
          GVariant      *state,
          gpointer       user_data)
{
    g_debug ("Requesting window be closed");
    gtk_window_close (GTK_WINDOW (user_data));
}

/*
 * on_fullscreen:
 * @action: A pointer to the object that originated the callback. In this
 * instance a #GSimpleAction, which will be added to the #GAction menu
 * @parameter: arg_data from the #GActionEntry
 * @user_data: User data set when the signal handler was connected, in this
 * instance it should be a #GtkApplication
 *
 * An action wrapper to toggle the window to fullscreen.
 */
static void
on_fullscreen (GSimpleAction *action,
               GVariant      *parameter,
               gpointer       user_data)
{
    gpod_window_toggle_fullscreen (GPOD_WINDOW (user_data));
}

const GActionEntry window_entries[] =
{
    { "close", on_close },
    { "fullscreen", on_fullscreen },
};

/*
 * gpod_window_set_fullscreen:
 * @self: An instance of #GpodWindow
 *
 * Sets the @self instance to fullscreen mode.
 */
void
gpod_window_toggle_fullscreen (GpodWindow *self)
{
    GPOD_IS_WINDOW (self);

    GdkWindow *window = gtk_widget_get_window (GTK_WIDGET (self));
    GdkWindowState state = gdk_window_get_state (window);

    if (state & GDK_WINDOW_STATE_FULLSCREEN) {
        gtk_window_unfullscreen (GTK_WINDOW (self));
        g_debug ("Window is requesting to unfullscreen");
    } else {
        gtk_window_fullscreen (GTK_WINDOW (self));
        g_debug ("Window is requesting to fullscreen");
    }

    /*
     * DEBUG CODE:
     * Check new state, after request
     */
    state = gdk_window_get_state (window);

    if (state & GDK_WINDOW_STATE_FULLSCREEN) {
        g_debug ("Window is unfullscreen");
    } else {
        g_debug ("Window is fullscreen");
    }
}

/*
 * gpod_window_state_store:
 * @self: An instance of #GpodWindow
 *
 * Saves the `width`, `height`, `x`, `y`, `maximized` and `fullscreen` values
 * from from the #GpodWindowState struct into the gsettings:
 * PACKAGE_ID ".window-state", eg. tld.example.app.window-state
 */
static void
gpod_window_state_store (GpodWindow *self)
{
    GSettingsSchemaSource *source = g_settings_schema_source_get_default ();
    GSettings *settings;
    gboolean status;

    /* If window-state schema not found, can't save settings. However, not a
     * critical failure. No settings saved, instead the aplication will use the
     * defaulted initialized values in gpod_window_constructed()
     * and log a g_warning().
     */
    if (g_settings_schema_source_lookup (source, PACKAGE_ID ".window-state", FALSE) == NULL)
    {
        g_warning ("Unable to save window-state to GSettingsSchema: \"%s\"", PACKAGE_ID ".window-state");
        return;
    }

    settings = g_settings_new (PACKAGE_ID ".window-state");

    status = g_settings_set_int (settings, "width", self->state.width);

    if (status)
        g_debug ("WindowSate: Saved width      = %i", self->state.width);
    else
        g_debug ("WindowState: Could not save width");


    status = g_settings_set_int (settings, "height", self->state.height);

    if (status)
        g_debug ("WindowSate: Saved height     = %i", self->state.height);
    else
        g_debug ("WindowState: Could not save height");

    status = g_settings_set_int (settings, "x", self->state.x);

    if (status)
        g_debug ("WindowSate: Saved position-x = %i", self->state.height);
    else
        g_debug ("WindowState: Could not save postion-x");

    status = g_settings_set_int (settings, "y", self->state.y);

    if (status)
        g_debug ("WindowSate: Saved position-y = %i", self->state.height);
    else
        g_debug ("WindowState: Could not save positon-y");

    status = g_settings_set_boolean (settings, "maximized", self->state.maximized);

    if (status)
        g_debug ("WindowSate: Saved maximized  = %s", (self->state.maximized) ? "TRUE" : "FALSE");
    else
        g_debug ("WindowState: Could not save maximized");

    status = g_settings_set_boolean (settings, "fullscreen", self->state.fullscreen);

    if (status)
        g_debug ("WindowSate: Saved fullscreen = %s", (self->state.fullscreen) ? "TRUE" : "FALSE");
    else
        g_debug ("WindowState: Could not save fullscreen");
}

/*
 * gpod_window_state_load:
 * @self: An instance of #GpodWindow
 *
 * Loads the `width`, `height`, `x`, `y`, `maximized` and `fullscreen` values
 * from gsettings into the #GpodWindowState struct, located in the
 * #GpodWindow struct.
 */
static void
gpod_window_state_load (GpodWindow *self)
{
    GSettingsSchemaSource *source = g_settings_schema_source_get_default ();
    GSettings *settings;

    /* If window-state schema not found, can't load settings. However, not a
     * critical failure. Use the initialized values in gpod_window_constructed()
     * and log a g_warning().
     */
    if (g_settings_schema_source_lookup (source, PACKAGE_ID ".window-state", FALSE) == NULL)
    {
        g_warning ("Unable to load GSettingsSchema: \"%s\", using default window state", PACKAGE_ID ".window-state");
        return;
    }

    settings = g_settings_new (PACKAGE_ID ".window-state");

    self->state.width = g_settings_get_int (settings, "width");
    g_debug ("WindowSate: Loaded width      = %i", self->state.width);

    self->state.height = g_settings_get_int (settings, "height");
    g_debug ("WindowSate: Loaded height     = %i", self->state.height);

    self->state.x = g_settings_get_int (settings, "x");
    g_debug ("WindowSate: Loaded position-x = %i", self->state.x);

    self->state.y = g_settings_get_int (settings, "y");
    g_debug ("WindowSate: Loaded position-y = %i", self->state.y);

    self->state.maximized = g_settings_get_boolean (settings, "maximized");
    g_debug ("WindowSate: Loaded maximized  = %s", (self->state.maximized) ? "TRUE" : "FALSE");

    self->state.fullscreen = g_settings_get_boolean (settings, "fullscreen");
    g_debug ("WindowSate: Loaded fullscreen = %s", (self->state.fullscreen) ? "TRUE" : "FALSE");
}

/*
 * gpod_window_state_set:
 * @user_data: Data passed into the function via g_timeout_add(), in this case
 * an instance of #GpodWindow
 *
 * Saves the information from the current window into the WindowState. Is called
 * from gpod_window_configure_event() and gpod_window_window_state_event().
 *
 * Returns: #gboolean; Sets %FALSE when the function does what it was going todo
 * (save window state) and set timeout in to 0.
 */
static gboolean
gpod_window_state_set (gpointer user_data)
{
    GpodWindow *self = GPOD_WINDOW (user_data);
    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    GdkWindow *window = gtk_widget_get_window (GTK_WIDGET (self));
    GdkWindowState state = gdk_window_get_state (window);

    self->state.maximized  = (state & GDK_WINDOW_STATE_MAXIMIZED) ? 1 : 0;
    self->state.fullscreen = (state & GDK_WINDOW_STATE_FULLSCREEN) ? 1 : 0;

    if (self->state.maximized || self->state.fullscreen)
    {
        priv->timeout_id = 0;
        return FALSE;
    }

    gtk_window_get_size (GTK_WINDOW (self),
                         &self->state.width,
                         &self->state.height);

    // TODO: Make saving postion x and y work in Wayland?
    gtk_window_get_position (GTK_WINDOW (self),
                             &self->state.x,
                             &self->state.y);

    priv->timeout_id = 0;

    return FALSE;
}

/*
 * gpod_window_configure_event:
 * @widget: An instance of #GpodWindow
 * @event: The current state of the window as a #GdkEventWindowState
 *
 * Saves the window state/geometry when it changes. Calls
 * gpod_window_state_set().
 *
 * Returns: The #gboolean return value (retval); %TRUE the event happened or
 * %FALSE the event did not happen
 */
static gboolean
gpod_window_configure_event (GtkWidget         *widget,
                             GdkEventConfigure *event)
{
    GpodWindow *self = GPOD_WINDOW (widget);
    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);
    gboolean retval;

    if (priv->timeout_id != 0)
    {
        g_source_remove (priv->timeout_id);
        priv->timeout_id = 0;
    }

    priv->timeout_id = g_timeout_add (STATE_TIMEOUT,
                                      gpod_window_state_set,
                                      self);

    retval = GTK_WIDGET_CLASS (gpod_window_parent_class)->configure_event (widget, event);

    return retval;
}

/*
 * gpod_window_destroy:
 * @widget: An instance of #GpodWindow
 * @event: The current state of the window as a #GdkEventWindowState
 *
 * Saves the window state/geometry when it changes, a.k.a if the window is
 * maximized or fullscreen. Calls gpod_window_state_set().
 *
 * Returns: The #gboolean return value (retval); %TRUE the event happened or
 * %FALSE the event did not happen
 */
static gboolean
gpod_window_window_state_event (GtkWidget           *widget,
                                GdkEventWindowState *event)
{
    GpodWindow *self = GPOD_WINDOW (widget);
    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);
    gboolean retval;

    if (priv->timeout_id != 0)
    {
        g_source_remove (priv->timeout_id);
        priv->timeout_id = 0;
    }

    priv->timeout_id = g_timeout_add (STATE_TIMEOUT,
                                      gpod_window_state_set,
                                      self);

    retval = GTK_WIDGET_CLASS (gpod_window_parent_class)->window_state_event (widget, event);

    return retval;
}

/*
 * gpod_window_constructed:
 * @object: An instance of #GpodWindow
 *
 * The constructed virtual function sets first-run settings for the
 * #GpodWindow object.
 *
 * It sets:
 *  * Window dimensions
 *  * Window postion
 *  * Window maximized
 *  * Window fullscreen
 */
static void
gpod_window_constructed (GObject *object)
{
    GpodWindow *self = GPOD_WINDOW (object);
    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    self->state.width      = -1;
    self->state.height     = -1;
    self->state.x          =  0;
    self->state.y          =  0;
    self->state.maximized  =  FALSE;
    self->state.fullscreen =  FALSE;

    gpod_window_state_load (self);

    gtk_window_set_default_size (GTK_WINDOW (self),
                                 self->state.width,
                                 self->state.height);

    gtk_window_set_position (GTK_WINDOW (self), GTK_WIN_POS_MOUSE);

    if (self->state.x && self->state.y)
        gtk_window_move (GTK_WINDOW (self),
                         self->state.x,
                         self->state.y);

    if (self->state.maximized)
        gtk_window_maximize (GTK_WINDOW (self));

    if (self->state.fullscreen)
        gtk_window_fullscreen (GTK_WINDOW (self));

    /*
     * Signals
     */
    /* Window */
    g_signal_connect (priv->notification_database_error_button_reload,
                      "clicked",
                      G_CALLBACK (on_notifcation_database_error_button_reload),
                      self);

    g_signal_connect (priv->notification_database_error_button_close,
                      "clicked",
                      G_CALLBACK (on_notifcation_database_error_button_close),
                      self);

    /* Feed Manager */
    g_signal_connect (priv->feed_manager,
                      "loading",
                      G_CALLBACK (on_notifcation_loading_show),
                      self);

    g_signal_connect (priv->feed_manager,
                      "loading-finished",
                      G_CALLBACK (on_notifcation_loading_hide),
                      self);

    g_signal_connect (priv->feed_manager,
                      "database-error",
                      G_CALLBACK (on_notifcation_database_error_show),
                      self);

    g_signal_connect (priv->feed_manager,
                      "no-results",
                      G_CALLBACK (on_notifcation_no_results),
                      self);

    /*
     * Setup views
     * Load feeds, etc...
     */
    load_library_view (self);

    G_OBJECT_CLASS (gpod_window_parent_class)->constructed (object);
}

/*
 * gpod_window_finalize:
 * @object: An instance of #GpodWindow
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_window_finalize (GObject *object)
{
    GpodWindow *self = GPOD_WINDOW (object);
    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    gpod_window_state_store (self);

    g_clear_object (&priv->headerbar);
    g_clear_object (&priv->player);

    G_OBJECT_CLASS (gpod_window_parent_class)->finalize(object);
}

/*
 * gpod_window_init:
 * @self: An instance of #GpodWindow
 *
 * Initializes various aspects of the window. Anything the window would need
 * before becoming presented should be done here.
 */
static void
gpod_window_init (GpodWindow *self)
{
    GApplication *application = g_application_get_default ();
    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);
    GtkBuilder *builder = gtk_builder_new ();
    GError *error = NULL;

    const gchar *fullscreen_accels[] =
    {
        "F11",
        "<Primary>f",
        NULL
    };

    /* Init window.ui template */
    gtk_widget_init_template (GTK_WIDGET (self));

    /* Set window title */
    gtk_window_set_title (GTK_WINDOW (self), _("Podcast"));

    /* Add HeaderBar */
    if (!gtk_builder_add_from_resource (builder, PACKAGE_RESOURCES "/ui/headerbar.ui", &error))
    {
        g_warning ("%s", error->message);
        g_clear_error (&error);
    }
    else
    {
        priv->headerbar = GTK_WIDGET (g_object_ref (gtk_builder_get_object (builder, "headerbar")));
        gtk_window_set_titlebar (GTK_WINDOW (self), priv->headerbar);
    }

    /* Add Player */
    if (!gtk_builder_add_from_resource (builder, PACKAGE_RESOURCES "/ui/player.ui", &error))
    {
        g_warning ("%s", error->message);
        g_clear_error (&error);
    }
    else
    {
        priv->player = GTK_WIDGET (g_object_ref (gtk_builder_get_object (builder, "player")));
        gtk_container_add (GTK_CONTAINER (priv->actionbar), priv->player);
    }

    /* Add library view */
    priv->library_view = gpod_library_view_new ();

    gtk_stack_add_named (GTK_STACK (priv->stack),
                         GTK_WIDGET (priv->library_view),
                         "library");

    /* Add feed view */
    //priv->feed_view = gpod_feed_view_new ();

    //gtk_stack_add_named (GTK_STACK (priv->stack),
    //                     GTK_WIDGET (priv->feed_view),
    //                     "feed");


    /* Add episode view */

    /* Add empty states */
    if (!gtk_builder_add_from_resource (builder, PACKAGE_RESOURCES "/ui/views/empty.ui", &error))
    {
        g_warning ("%s", error->message);
        g_clear_error (&error);
    }
    else
    {
        gtk_stack_add_named (GTK_STACK (priv->stack),
                             GTK_WIDGET (g_object_ref (gtk_builder_get_object (builder, "empty_view_feeds"))),
                             "empty_view_feeds");

        gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "empty_view_feeds");

        gtk_stack_add_named (GTK_STACK (priv->stack),
                             GTK_WIDGET (g_object_ref (gtk_builder_get_object (builder, "empty_view_search"))),
                             "empty_view_search");
    }

    /* Clean-up */
    g_clear_object (&builder);

    /* Add keybind */
    g_action_map_add_action_entries (G_ACTION_MAP (self),
                                     window_entries, G_N_ELEMENTS (window_entries),
                                     self);

    gpod_application_set_accelerator (GTK_APPLICATION (application), "win.close", "<Primary>w");
    gpod_application_set_accelerators (GTK_APPLICATION (application), "win.fullscreen", fullscreen_accels);

    /* Clean-up */
    if (error != NULL)
        g_error_free (error);
}

static void
gpod_window_get_property (GObject    *object,
                          guint       property_id,
                          GValue     *value,
                          GParamSpec *pspec)
{
    GpodWindow *self = GPOD_WINDOW (object);
    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    switch (property_id)
    {
        case PROP_FEED_MANAGER:
            g_value_set_object (value, priv->feed_manager);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
gpod_window_set_property (GObject      *object,
                          guint         property_id,
                          const GValue *value,
                          GParamSpec   *pspec)
{
    GpodWindow *self = GPOD_WINDOW (object);
    GpodWindowPrivate *priv = gpod_window_get_instance_private (self);

    switch (property_id)
    {
        case PROP_FEED_MANAGER:
            priv->feed_manager = g_value_get_object (value);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

/*
 * gpod_window_class_init:
 * @klass: An instance of #GpodWindowClass
 *
 * Initializes the various virtural functions in #GpodWindowClass. In
 * addtition it connects the various #GtkWidgets from the gresource file using
 * gtk_widget_class_set_template_from_resource().
 */
static void
gpod_window_class_init (GpodWindowClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

    object_class->constructed = gpod_window_constructed;
    object_class->finalize = gpod_window_finalize;
    object_class->get_property = gpod_window_get_property;
    object_class->set_property = gpod_window_set_property;

    widget_class->configure_event = gpod_window_configure_event;
    widget_class->window_state_event = gpod_window_window_state_event;

    /**
    * GpodWindow::feed-manager:
    *
    * A weak reference to the application's #GpodFeedManager instance.
    */
    properties[PROP_FEED_MANAGER] =
        g_param_spec_object ("feed-manager",
                             "Feed Manager of this window's application",
                             "The Feed manager of the window's application",
                             GPOD_TYPE_FEED_MANAGER,
                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);

    g_object_class_install_properties (object_class,
                                       LAST_PROP,
                                       properties);

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 PACKAGE_RESOURCES "/ui/window.ui");

    gtk_widget_class_bind_template_child_private (widget_class, GpodWindow, box);
    gtk_widget_class_bind_template_child_private (widget_class, GpodWindow, overlay);
    gtk_widget_class_bind_template_child_private (widget_class, GpodWindow, notification_loading);
    gtk_widget_class_bind_template_child_private (widget_class, GpodWindow, notification_database_error);
    gtk_widget_class_bind_template_child_private (widget_class, GpodWindow, notification_database_error_button_reload);
    gtk_widget_class_bind_template_child_private (widget_class, GpodWindow, notification_database_error_button_close);
    gtk_widget_class_bind_template_child_private (widget_class, GpodWindow, stack);
    gtk_widget_class_bind_template_child_private (widget_class, GpodWindow, actionbar);
}

/**
 * gpod_window_new:
 * @application: A #GtkApplication instance
 *
 * Creates a new #GpodWindow, a sub-object of #GtkApplicationWindow.
 *
 * ## Usage
 * |[
 * GpodWindow *window;
 *
 * window = gpod_window_new (application);
 * ]|
 *
 * When creating a gpod_window_new() instance it's recommended to set the
 * instance into a variable inside the #GpodApplication struct. The
 * window could also be added to a #GList, so multiple windows can belong to
 * the application.
 *
 * Returns: #GpodWindow; a sub-object of #GtkApplicationWindow
 */
GpodWindow *
gpod_window_new (GpodApplication *application,
                 GpodFeedManager *feed_manager)
{
    g_return_val_if_fail (GPOD_IS_APPLICATION (application), NULL);
    g_return_val_if_fail (GPOD_IS_FEED_MANAGER (feed_manager), NULL);

    return g_object_new (GPOD_TYPE_WINDOW,
                         "application", application,
                         "feed-manager", feed_manager,
                         NULL);
}

